/****************************************************************************
 * OBIDMS_column_char functions                                              *
 ****************************************************************************/

/**
 * @file obidsmcolumn_char.c
 * @author Celine Mercier
 * @date August 10th 2015
 * @brief Functions handling OBIColumns containing data with the OBIType OBI_CHAR.
 */


#include <stdlib.h>
#include <stdio.h>

#include "obidmscolumn.h"
#include "obitypes.h"
#include "obierrno.h"
#include "obidebug.h"


#define DEBUG_LEVEL 0


/**********************************************************************
 *
 * D E F I N I T I O N   O F   T H E   P U B L I C   F U N C T I O N S
 *
 **********************************************************************/

int obi_column_set_char_with_elt_idx(OBIDMS_column_p column, size_t line_nb, size_t element_idx, obichar_t* value)
{
	// Update lines used
	if ((line_nb+1) > (column->header)->lines_used)
		(column->header)->lines_used = line_nb+1;
	*(((obichar_t*) (column->data)) + (line_nb * ((column->header)->nb_elements_per_line)) + element_idx) = value[0];
	return 0;
}


obichar_t* obi_column_get_char_with_elt_idx(OBIDMS_column_p column, size_t line_nb, size_t element_idx)
{
	return ((obichar_t*) (column->data)) + (line_nb * ((column->header)->nb_elements_per_line)) + element_idx;
}


int obi_column_set_char_with_elt_name(OBIDMS_column_p column, size_t line_nb, char* element_name, obichar_t* value)
{
	size_t element_idx;

	if (!strcmp(element_name, "\0"))	// element name is empty
	{
		if (obi_column_get_nb_elements_per_line(column) == 1)	// check that there is only one element per line
			element_idx = 0;
		else													// there is more than one element per line
		{
			obi_set_errno(OBICOL_UNKNOWN_ERROR);
			obidebug(1, "\nAn element name must be specified");
        	return -1;
		}
	}
    else
    {
		element_idx = obi_column_get_element_index_from_name(column, element_name);
		if (element_idx == -1)
			return -1;
    }

	obi_column_set_char_with_elt_idx(column, line_nb, element_idx, value);

	return 0;
}


obichar_t* obi_column_get_char_with_elt_name(OBIDMS_column_p column, size_t line_nb, char* element_name)
{
	size_t element_idx;

	if (!strcmp(element_name, "\0"))	// element name is empty
	{
		if (obi_column_get_nb_elements_per_line(column) == 1)	// check that there is only one element per line
			element_idx = 0;
		else													// there is more than one element per line
		{
			obi_set_errno(OBICOL_UNKNOWN_ERROR);
			obidebug(1, "\nAn element name must be specified");
        	return NULL;
		}
	}
    else
    {
		element_idx = obi_column_get_element_index_from_name(column, element_name);
		if (element_idx == -1)
			return NULL;
    }

	return obi_column_get_char_with_elt_idx(column, line_nb, element_idx);
}
